package com.gihon.component.web.auth.impl;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

import javax.annotation.PostConstruct;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingClass;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.http.HttpMethod;
import org.springframework.stereotype.Service;
import org.springframework.util.AntPathMatcher;

import com.fasterxml.jackson.core.type.TypeReference;
import com.gihon.component.entity.rbac.GihonButton;
import com.gihon.component.entity.rbac.GihonMenu;
import com.gihon.component.entity.rbac.GihonModule;
import com.gihon.component.entity.rbac.GihonRole;
import com.gihon.component.properties.GihonCommonProperties;
import com.gihon.component.properties.RedisConstants;
import com.gihon.component.service.GihonButtonService;
import com.gihon.component.service.GihonMenuService;
import com.gihon.component.service.GihonModuleService;
import com.gihon.component.service.GihonRoleButtonService;
import com.gihon.component.service.GihonRoleService;
import com.gihon.component.util.JacksonJsonUtils;
import com.gihon.component.web.auth.AuthService;
import com.gihon.component.web.auth.AuthUser;
import com.gihon.component.web.service.TokenService;
import com.gihon.component.web.vo.MenuVO;
import com.gihon.component.web.vo.PermissionVO;
import com.gihon.component.web.vo.TokenVal;

import lombok.extern.slf4j.Slf4j;

/**
 * 动态鉴权
 * 非Security
 * @author baihe
 *
 */
@ConditionalOnMissingClass("org.springframework.security.core.userdetails.UserDetails")
@Slf4j
@Service
public class AuthServiceImpl implements AuthService{

    @Autowired
    private TokenService tokenService;

    @Autowired
    private StringRedisTemplate stringRedisTemplate;

    @Autowired
    private GihonCommonProperties gihonCommonProperties;
    
    @Autowired
    private GihonModuleService gihonModuleService;
    
    @Autowired
    private GihonMenuService gihonMenuService;
    
    @Autowired
    private GihonButtonService gihonButtonService;
    
    @Autowired
    private GihonRoleButtonService gihonRoleButtonService;
    
    @Autowired
    private GihonRoleService gihonRoleService;
   
    private AntPathMatcher antMatcher = new AntPathMatcher();
    
    /**
     * 通过请求地址和当前用户权限 获取请求地址需要的权限比较 忽略平台类型
     * 
     * @param uri
     * @param token
     * @param checkResult
     * @return
     */
    public boolean checkPermission(String uri, int moduleType, String token, AuthUser checkResult) {
        TokenVal tv = tokenService.getTokenVal(token);
        if (tv == null || tv.getModuleType() != moduleType) {
            return false;
        }
        // 获取全部的角色
        List<String> role = checkResult.getRoleList();

        // 获取全部的权限列表匹配
        // 用权限匹配角色,如果没有角色那么就通过
        List<String> needRole = getPermission(uri);
        // Null 是没有配置权限只要登录即可
        if (needRole == null) {
            return true;
        } else if (role == null||role.isEmpty()) {
            return false;
        }
        // Empty是配置权限但是没有角色 只有管理员才行 @See PermissionServiceImpl#cloneCopyMenu
        return needRole.stream().anyMatch(r -> role.contains(r));
    }

    /**
     * 从redis缓存中读取权限
     * 
     * @return
     */
    public List<String> getPermission(String uri) {
        String[] uris = uri.split(RedisConstants.SEP);

        Comparator<String> c = antMatcher.getPatternComparator("/**");

        List<PermissionVO> list = getPermissionList();
        // 排序
        HttpMethod m = HttpMethod.valueOf(uris[0]);
        list = list.stream().filter(p -> m.matches(p.getMethod())).sorted((p, p2) -> {
            return c.compare(p.getUrl(), p2.getUrl());
        }).collect(Collectors.toList());

        for (PermissionVO vo : list) {
//			RequestMatcher matcher = new AntPathRequestMatcher(vo.getUrl(),vo.getMethod());
            if (antMatcher.match(vo.getUrl(), uris[1])) {
                return vo.getRoleList();
            }
        }
        return null;
    }

    /**
     * 从redis缓存中读取权限
     * @See AuthService.init()
     * @return
     */
    public List<PermissionVO> getPermissionList() {
        String hashKey = gihonCommonProperties.getModuleCode() + RedisConstants.SEP + gihonCommonProperties.getModuleType();
        String result = (String) stringRedisTemplate.opsForHash().get(RedisConstants.PERMISSN_REDIS, hashKey);
        if (StringUtils.isBlank(result)) {
            return null;
        }
        TypeReference<List<PermissionVO>> r = new TypeReference<List<PermissionVO>>() {
        };
        return JacksonJsonUtils.readObject(result, r);
    }


    // TODO 定时刷新或者AOP通知
    @PostConstruct
    public void init() {
        try {

            GihonModule module = gihonModuleService.getModuleByCode(gihonCommonProperties.getModuleCode());
            if (module == null) {
                return;
            }
            List<GihonMenu> menuList = gihonMenuService.getAllMenu(module.getId(),false);
            
            List<MenuVO> menus = new ArrayList<>();
            
            MenuVO root = new MenuVO();
            root.setMenuCode(gihonCommonProperties.getModuleCode());
            root.setMenuId(-1L);
            
            menus.add(root);
            
            MenuVO mp = new MenuVO();
           
            menuList.forEach(m -> {
                MenuVO mv = new MenuVO();
                mv.setMenuCode(m.getMenuCode());
                mv.setMenuId(m.getId());
                menus.add(mv);
//                mp.setMenuId(m.getParentId());
//                int index = menus.indexOf(mp);
//                if (index > 0) {
//                    menus.get(index).addMenu(mp);
//                }
            });
            mp = new MenuVO();
            List<GihonButton> buttonList = gihonButtonService.getMenuButton(menuList.stream().map(m -> m.getId()).collect(Collectors.toList()),false);
            List<PermissionVO> plist = new ArrayList<PermissionVO>();
            for (GihonButton btn : buttonList) {
                List<Long> rbtnList = gihonRoleButtonService.getRoleListByBtn(btn.getId());
                PermissionVO pvo = new PermissionVO();
                pvo.setUrl(btn.getPath());
                pvo.setMethod(btn.getMethod());
                pvo.setBtnCode(btn.getButtonCode());
                pvo.setBtnName(btn.getButtonName());
                if (!rbtnList.isEmpty()) {
                    List<GihonRole> rl = gihonRoleService.getRoleListByIds(rbtnList);
                    pvo.setRoleList(rl.stream().map(r -> r.getCompanyId()+RedisConstants.SEP+r.getRoleCode()).collect(Collectors.toList()));
                }else {
                    pvo.setRoleList(new ArrayList<>(0));
                }
                mp.setMenuId(btn.getMenuId());
                int index = menus.indexOf(mp);
                if (index > 0) {
                    menus.get(index).addPermission(pvo);
                }
                if (StringUtils.isBlank(btn.getPath())) {
                    continue;
                }
                plist.add(pvo);
            }
            //用于拦截器的访问权限按钮
            stringRedisTemplate.opsForHash().put(RedisConstants.PERMISSN_REDIS,
                    gihonCommonProperties.getModuleCode() + RedisConstants.SEP + gihonCommonProperties.getModuleType(), JacksonJsonUtils.writeObject(plist));
            //用于拦截器的访问权限按钮
            stringRedisTemplate.opsForHash().put(RedisConstants.MENU_TREE_REDIS,
                    gihonCommonProperties.getModuleCode() + RedisConstants.SEP + gihonCommonProperties.getModuleType(), JacksonJsonUtils.writeObject(menus));
        } catch (Exception e) {
            log.error("初始化AuthService失败:", e);
        }
    }
}
