package com.gihon.component.web.config;

import static com.gihon.component.util.DateUtils.DEFAULT_DATE_TIME_FORMAT;
import static com.gihon.component.util.DateUtils.FULL_TIME_FORMATTER;
import static com.gihon.component.util.DateUtils.TIME_FORMATTER;
import static com.gihon.component.util.DateUtils.YEAR_MONTH_DAY_FORMATTER;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.nio.charset.Charset;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;

import javax.net.ssl.SSLContext;

import org.apache.commons.lang3.StringUtils;
import org.apache.http.conn.ssl.NoopHostnameVerifier;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.ssl.SSLContextBuilder;
import org.apache.http.ssl.TrustStrategy;
import org.springframework.boot.autoconfigure.condition.ConditionalOnClass;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.boot.autoconfigure.jackson.Jackson2ObjectMapperBuilderCustomizer;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;
import org.springframework.core.convert.converter.Converter;
import org.springframework.http.client.HttpComponentsClientHttpRequestFactory;
import org.springframework.http.converter.HttpMessageConverter;
import org.springframework.http.converter.StringHttpMessageConverter;
import org.springframework.http.converter.json.Jackson2ObjectMapperBuilder;
import org.springframework.http.converter.json.MappingJackson2HttpMessageConverter;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.cors.UrlBasedCorsConfigurationSource;
import org.springframework.web.filter.CorsFilter;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.json.JsonReadFeature;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.MapperFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializationFeature;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import com.fasterxml.jackson.datatype.jsr310.deser.LocalDateDeserializer;
import com.fasterxml.jackson.datatype.jsr310.deser.LocalDateTimeDeserializer;
import com.fasterxml.jackson.datatype.jsr310.deser.LocalTimeDeserializer;
import com.fasterxml.jackson.datatype.jsr310.ser.LocalDateSerializer;
import com.fasterxml.jackson.datatype.jsr310.ser.LocalDateTimeSerializer;
import com.fasterxml.jackson.datatype.jsr310.ser.LocalTimeSerializer;

@Configuration
public class CommonWebConfig implements WebMvcConfigurer {

	/**
	 * 此类创建ObjectMapper
	 * 解决Date,LocalDateTime,Long的解析
	 * @return
	 */
	@Bean
	public Jackson2ObjectMapperBuilderCustomizer jackson2ObjectMapperBuilderCustomizer() {
		return builder -> builder
				.serializerByType(LocalDateTime.class, new LocalDateTimeSerializer(FULL_TIME_FORMATTER))
				.serializerByType(LocalDate.class, new LocalDateSerializer(YEAR_MONTH_DAY_FORMATTER))
				.serializerByType(LocalTime.class, new LocalTimeSerializer(TIME_FORMATTER))
				.deserializerByType(LocalDateTime.class, new LocalDateTimeDeserializer(FULL_TIME_FORMATTER))
				.deserializerByType(LocalDate.class, new LocalDateDeserializer(YEAR_MONTH_DAY_FORMATTER))
				.deserializerByType(LocalTime.class, new LocalTimeDeserializer(TIME_FORMATTER))
				.serializerByType(Long.class, ToStringSerializer.instance)
				.serializerByType(BigInteger.class, ToStringSerializer.instance)
				.serializerByType(BigDecimal.class, ToStringSerializer.instance)
				.dateFormat(new SimpleDateFormat(DEFAULT_DATE_TIME_FORMAT))
				.featuresToEnable(MapperFeature.PROPAGATE_TRANSIENT_MARKER)// 忽略 transient 修饰的属性
				.featuresToEnable(JsonReadFeature.ALLOW_UNESCAPED_CONTROL_CHARS.mappedFeature())
				//该特性决定parser是否允许JSON字符串包含非引号控制字符（值小于32的ASCII字符，包含制表符和换行符）。
				// 如果该属性关闭，则如果遇到这些字符，则会抛出异常。JSON标准说明书要求所有控制符必须使用引号，因此这是一个非标准的特性
//              .featuresToEnable(JsonParser.Feature.ALLOW_UNQUOTED_CONTROL_CHARS)
//				.featuresToEnable(JsonParser.Feature.ALLOW_BACKSLASH_ESCAPING_ANY_CHARACTER)// 忽略不能转移的字符
				.featuresToEnable(JsonReadFeature.ALLOW_BACKSLASH_ESCAPING_ANY_CHARACTER.mappedFeature())
				.featuresToEnable(JsonParser.Feature.ALLOW_SINGLE_QUOTES)// 单引号处理
				.featuresToDisable(SerializationFeature.WRITE_DATES_AS_TIMESTAMPS)
				.featuresToDisable(SerializationFeature.FAIL_ON_EMPTY_BEANS)
				.featuresToDisable(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES)// 忽略未知字段
                .serializationInclusion(JsonInclude.Include.ALWAYS);
	}

	@Bean
	@Primary
	@ConditionalOnClass(ObjectMapper.class)
	@ConditionalOnMissingBean({ ObjectMapper.class })
	public ObjectMapper jacksonObjectMapper(Jackson2ObjectMapperBuilder builder) {
		ObjectMapper objectMapper = builder.createXmlMapper(false).build();
		objectMapper.setLocale(Locale.CHINA);
		return objectMapper;
	}

	@Bean
	@Primary
	public RestTemplate restTemplate(ObjectMapper objectMapper) throws Exception {

		SSLContext sslContext = new SSLContextBuilder().loadTrustMaterial(null, new TrustStrategy() {
			@Override
			public boolean isTrusted(X509Certificate[] arg0, String arg1) throws CertificateException {
				return true;
			}
		}).build();

		SSLConnectionSocketFactory csf = new SSLConnectionSocketFactory(sslContext, new String[] { "TLSv1" }, null,
				NoopHostnameVerifier.INSTANCE);
		CloseableHttpClient httpClient2 = HttpClients.custom().setSSLSocketFactory(csf).build();
		HttpComponentsClientHttpRequestFactory requestFactory = new HttpComponentsClientHttpRequestFactory();
		requestFactory.setHttpClient(httpClient2);
		requestFactory.setReadTimeout(600000);// 超时时间增加为10分钟
		requestFactory.setConnectTimeout(60000);// 连接时间增加为1分钟
		RestTemplate restTemplate = new RestTemplate(requestFactory);
		// 使用 utf-8 编码集的 conver 替换默认的 conver（默认的 string conver 的编码集为
		// "ISO-8859-1"）
		List<HttpMessageConverter<?>> messageConverters = restTemplate.getMessageConverters();
		Iterator<HttpMessageConverter<?>> iterator = messageConverters.iterator();
		while (iterator.hasNext()) {
			HttpMessageConverter<?> converter = iterator.next();
			if (converter instanceof StringHttpMessageConverter) {
				((StringHttpMessageConverter) converter).setDefaultCharset(Charset.forName("UTF-8"));
			} else if (converter instanceof MappingJackson2HttpMessageConverter) {
				((MappingJackson2HttpMessageConverter) converter).setObjectMapper(objectMapper);
			}
		}
		return restTemplate;
	}
	 /**
     * attention:简单跨域就是GET，HEAD和POST请求，但是POST请求的"Content-Type"只能是application/x-www-form-urlencoded, multipart/form-data 或 text/plain
     * 反之，就是非简单跨域，此跨域有一个预检机制，说直白点，就是会发两次请求，一次OPTIONS请求，一次真正的请求
     */
    @Bean
    public CorsFilter corsFilter() {
        final UrlBasedCorsConfigurationSource source = new UrlBasedCorsConfigurationSource();
        final org.springframework.web.cors.CorsConfiguration config = new org.springframework.web.cors.CorsConfiguration();
        // 允许cookies跨域
        config.setAllowCredentials(true);
        // #允许向该服务器提交请求的URI，*表示全部允许，在SpringMVC中，如果设成*，会自动转成当前请求头中的Origin
        config.addAllowedOrigin("*");
        // #允许访问的头信息,*表示全部
        config.addAllowedHeader("*");
        // 预检请求的缓存时间（秒），即在这个时间段里，对于相同的跨域请求不会再预检了
        config.setMaxAge(18000L);
        // 允许提交请求的方法，*表示全部允许
        config.addAllowedMethod("OPTIONS");
        config.addAllowedMethod("HEAD");
        // 允许Get的请求类型
        config.addAllowedMethod("GET");
        config.addAllowedMethod("PUT");
        config.addAllowedMethod("POST");
        config.addAllowedMethod("DELETE");
        config.addAllowedMethod("PATCH");
        source.registerCorsConfiguration("/**", config);
        return new CorsFilter(source);
    }
    
    /********普通请求参数中的时间转换******/
    @Bean
    public StringToLocalDateTimeConverter localDateTimeConverter() {
        return new StringToLocalDateTimeConverter();
    }

    @Bean
    public StringToLocalDateConverter localDateConverter() {
        return new StringToLocalDateConverter();
    }

    @Bean
    public StringToLocalTimeConverter localTimeConverter() {
        return new StringToLocalTimeConverter();
    }

    private static class StringToLocalDateTimeConverter implements Converter<String, LocalDateTime> {
        @Override
        public LocalDateTime convert(String source) {
            try {
                if(!StringUtils.isBlank(source)) {
                    if(source.length()==10) {
                        return LocalDateTime.of(LocalDate.parse(source, YEAR_MONTH_DAY_FORMATTER),LocalTime.MIN);
                    }else {
                        return LocalDateTime.parse(source,FULL_TIME_FORMATTER);
                    }
                }
            }catch (Exception e) {
               
            }
            return null;
        }
    }

    private static class StringToLocalDateConverter implements Converter<String, LocalDate> {
        @Override
        public LocalDate convert(String source) {
            if(!StringUtils.isBlank(source)) {
                try {
                    return LocalDate.parse(source, YEAR_MONTH_DAY_FORMATTER);
                }catch (Exception e) {
                }
            }
            return null;
        }
    }

    private static class StringToLocalTimeConverter implements Converter<String, LocalTime> {
        @Override
        public LocalTime convert(String source) {
            if(!StringUtils.isBlank(source)) {
                try {
                    return LocalTime.parse(source, TIME_FORMATTER);
                }catch (Exception e) {
                }
            }
            return null;
        }
    }
}
